//
//    Copyright 2005, Thomas C. McDermott, N5EG
//    This file is part of SIGGEN - the Signal Generator program.
//
//    SIGGEN is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    SIGGEN is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SIGGEN, if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//


#pragma once

#include "USB_EZ_interface.h"
#include "DataDisplay.h"
#include "ReferenceSelection.h"
#include "FrequencySelector.h"
#include "NumentBox.h"

namespace SGENR1
{
	using namespace System;
	using namespace System::ComponentModel;
	using namespace System::Collections;
	using namespace System::Windows::Forms;
	using namespace System::Data;
	using namespace System::Drawing;
	using namespace System::IO;
	using namespace System::Reflection;
	using namespace System::Threading;

	/// <summary> 
	/// Summary for Form1
	///
	/// WARNING: If you change the name of this class, you will need to change the 
	///          'Resource File Name' property for the managed resource compiler tool 
	///          associated with all .resx files this class depends on.  Otherwise,
	///          the designers will not be able to interact properly with localized
	///          resources associated with this form.
	/// </summary>
	public __gc class Form1 : public System::Windows::Forms::Form
	{	
	public:
		Form1(void)
		{
			InitializeComponent();
			SGEN_Initialize();
		}
  
	protected:
		void Dispose(Boolean disposing)
		{
			if (disposing && components)
			{
				components->Dispose();
			}
			__super::Dispose(disposing);
		}

	private:
		Point p1,p2;					// saved mouse coordinates
		int MarkerMousePick;			// marker number picked by mouse (-1 = not picked)
		int txLevel;					// transmit level

		//Random* rand;

		System::Windows::Forms::Timer* timer1;	// periodic event to gather phase reading

		Rectangle rect;					// client size for render (given to us by Paint & Redraw)
		Rectangle scopeDisp;			// rectangular scope display area
		Rectangle polarBox;				// box holding the polar display

		String*	plotTitle;				// Plot Title (allowed to be empty)

		bool WorkerCollect;				// Work for VNA Thread to do

		VNADevice* VNA;					// Vector Network Analyzer hardware object
		String* StartUpPath;			// Directory path we started execution in

		Mutex*	DataBufMut;				// Control access to VNA return data buffer
		Thread* VNAWorkerThread;				// Thread for VNA data gathering
		ThreadStart* VNAWorkerThreadDelegate;	// Threadstart points to method

		FrequencySelector * Fs;			// Frequency Selector
		double TxFrequency;
		float TxLevel;
		int FrequencyIndex, LevelIndex;		// digit selectors on user display

		// Frequency digit Selector text - index into array prints a dash underneath that position
		static String* FreqDigitStr __gc[] = { S"-__", S"-___", S"-____", S"-_____", S"-______",
					S"-_______", S"-_________", S"-__________", S"-___________", S"-_____________",
					S"-______________", S"-_______________", S"-_________________",
					S"-__________________", S"-___________________"	 };	
		
	
	
	private: System::Windows::Forms::Label *  frequency;
	private: System::Windows::Forms::MainMenu *  mainMenu1;
	private: System::Windows::Forms::MenuItem *  menuItem1;
	private: System::Windows::Forms::MenuItem *  menuItem2;
	private: System::Windows::Forms::MenuItem *  menuItem3;

	private: System::Windows::Forms::Label *  OutputLevel;
	private: System::Windows::Forms::MenuItem *  SelRefmenuItem;
	private: System::Windows::Forms::ToolTip *  toolTip1;
	private: System::Windows::Forms::Label *  FrequencyDigit;
	private: System::Windows::Forms::Label *  LevelDigit;
	private: System::Windows::Forms::Button *  FreqUp;
	private: System::Windows::Forms::Button *  FreqDown;
	private: System::Windows::Forms::Button *  LevelDown;
	private: System::Windows::Forms::Button *  LevelUp;
	private: System::Windows::Forms::Button *  FreqIndexLeft;
	private: System::Windows::Forms::Button *  FreqIndexRight;
	private: System::Windows::Forms::Button *  LevelIndexRight;
	private: System::Windows::Forms::Button *  LevelIndexLeft;
	private: System::Windows::Forms::Label *  label1;
	private: System::Windows::Forms::Label *  label2;
	private: AnalogMeter::AnalogMeterControl *  analogMeterControl1;
	private: System::Windows::Forms::Label *  label3;
	private: System::Windows::Forms::Label *  label4;

	private: System::ComponentModel::IContainer *  components;

	private:
		/// <summary>
		/// Required designer variable.
		/// </summary>


		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		void InitializeComponent(void)
		{
			this->components = (new System::ComponentModel::Container());
			this->frequency = (new System::Windows::Forms::Label());
			this->mainMenu1 = (new System::Windows::Forms::MainMenu(this->components));
			this->menuItem1 = (new System::Windows::Forms::MenuItem());
			this->menuItem2 = (new System::Windows::Forms::MenuItem());
			this->menuItem3 = (new System::Windows::Forms::MenuItem());
			this->SelRefmenuItem = (new System::Windows::Forms::MenuItem());
			this->OutputLevel = (new System::Windows::Forms::Label());
			this->toolTip1 = (new System::Windows::Forms::ToolTip(this->components));
			this->FrequencyDigit = (new System::Windows::Forms::Label());
			this->LevelDigit = (new System::Windows::Forms::Label());
			this->FreqUp = (new System::Windows::Forms::Button());
			this->FreqDown = (new System::Windows::Forms::Button());
			this->LevelDown = (new System::Windows::Forms::Button());
			this->LevelUp = (new System::Windows::Forms::Button());
			this->FreqIndexLeft = (new System::Windows::Forms::Button());
			this->FreqIndexRight = (new System::Windows::Forms::Button());
			this->LevelIndexRight = (new System::Windows::Forms::Button());
			this->LevelIndexLeft = (new System::Windows::Forms::Button());
			this->label1 = (new System::Windows::Forms::Label());
			this->label2 = (new System::Windows::Forms::Label());
			this->label3 = (new System::Windows::Forms::Label());
			this->label4 = (new System::Windows::Forms::Label());
			this->analogMeterControl1 = (new AnalogMeter::AnalogMeterControl());
			this->SuspendLayout();
			// 
			// frequency
			// 
			this->frequency->Anchor = System::Windows::Forms::AnchorStyles::Bottom;
			this->frequency->BackColor = System::Drawing::Color::Black;
			this->frequency->Font = (new System::Drawing::Font(S"Courier New", 18, System::Drawing::FontStyle::Bold, System::Drawing::GraphicsUnit::Point, 
				(System::Byte)0));
			this->frequency->ForeColor = System::Drawing::Color::Aquamarine;
			this->frequency->Location = System::Drawing::Point(136, 339);
			this->frequency->Name = S"frequency";
			this->frequency->Size = System::Drawing::Size(312, 32);
			this->frequency->TabIndex = 0;
			this->frequency->Text = S"10,000,000.000000";
			this->frequency->TextAlign = System::Drawing::ContentAlignment::MiddleRight;
			this->toolTip1->SetToolTip(this->frequency, S"Double click to directly enter Frequency");
			this->frequency->DoubleClick += new System::EventHandler(this, &Form1::frequency_DoubleClick);
			// 
			// mainMenu1
			// 
			System::Windows::Forms::MenuItem* __mcTemp__1[] = new System::Windows::Forms::MenuItem*[2];
			__mcTemp__1[0] = this->menuItem1;
			__mcTemp__1[1] = this->menuItem3;
			this->mainMenu1->MenuItems->AddRange(__mcTemp__1);
			// 
			// menuItem1
			// 
			this->menuItem1->Index = 0;
			System::Windows::Forms::MenuItem* __mcTemp__2[] = new System::Windows::Forms::MenuItem*[1];
			__mcTemp__2[0] = this->menuItem2;
			this->menuItem1->MenuItems->AddRange(__mcTemp__2);
			this->menuItem1->Text = S"File";
			// 
			// menuItem2
			// 
			this->menuItem2->Index = 0;
			this->menuItem2->Text = S"Exit";
			this->menuItem2->Click += new System::EventHandler(this, &Form1::menuItem2_Click);
			// 
			// menuItem3
			// 
			this->menuItem3->Index = 1;
			System::Windows::Forms::MenuItem* __mcTemp__3[] = new System::Windows::Forms::MenuItem*[1];
			__mcTemp__3[0] = this->SelRefmenuItem;
			this->menuItem3->MenuItems->AddRange(__mcTemp__3);
			this->menuItem3->Text = S"Reference";
			// 
			// SelRefmenuItem
			// 
			this->SelRefmenuItem->Index = 0;
			this->SelRefmenuItem->Text = S"Select Reference";
			this->SelRefmenuItem->Click += new System::EventHandler(this, &Form1::SelRefmenuItem_Click);
			// 
			// OutputLevel
			// 
			this->OutputLevel->Anchor = System::Windows::Forms::AnchorStyles::Bottom;
			this->OutputLevel->BackColor = System::Drawing::Color::Black;
			this->OutputLevel->Font = (new System::Drawing::Font(S"Courier New", 18, System::Drawing::FontStyle::Bold, System::Drawing::GraphicsUnit::Point, 
				(System::Byte)0));
			this->OutputLevel->ForeColor = System::Drawing::Color::Aquamarine;
			this->OutputLevel->Location = System::Drawing::Point(560, 339);
			this->OutputLevel->Name = S"OutputLevel";
			this->OutputLevel->Size = System::Drawing::Size(104, 32);
			this->OutputLevel->TabIndex = 1;
			this->OutputLevel->Text = S"0.0";
			this->OutputLevel->TextAlign = System::Drawing::ContentAlignment::MiddleRight;
			this->toolTip1->SetToolTip(this->OutputLevel, S"Double Click to directly enter Level");
			this->OutputLevel->DoubleClick += new System::EventHandler(this, &Form1::OutputLevel_DoubleClick);
			// 
			// FrequencyDigit
			// 
			this->FrequencyDigit->Anchor = System::Windows::Forms::AnchorStyles::Bottom;
			this->FrequencyDigit->BackColor = System::Drawing::Color::Black;
			this->FrequencyDigit->Font = (new System::Drawing::Font(S"Courier New", 18, System::Drawing::FontStyle::Bold, System::Drawing::GraphicsUnit::Point, 
				(System::Byte)0));
			this->FrequencyDigit->ForeColor = System::Drawing::Color::Aquamarine;
			this->FrequencyDigit->Location = System::Drawing::Point(136, 371);
			this->FrequencyDigit->Name = S"FrequencyDigit";
			this->FrequencyDigit->Size = System::Drawing::Size(312, 16);
			this->FrequencyDigit->TabIndex = 2;
			this->FrequencyDigit->Text = S"-_______ ";
			this->FrequencyDigit->TextAlign = System::Drawing::ContentAlignment::MiddleRight;
			// 
			// LevelDigit
			// 
			this->LevelDigit->Anchor = System::Windows::Forms::AnchorStyles::Bottom;
			this->LevelDigit->BackColor = System::Drawing::Color::Black;
			this->LevelDigit->Font = (new System::Drawing::Font(S"Courier New", 18, System::Drawing::FontStyle::Bold, System::Drawing::GraphicsUnit::Point, 
				(System::Byte)0));
			this->LevelDigit->ForeColor = System::Drawing::Color::Aquamarine;
			this->LevelDigit->Location = System::Drawing::Point(560, 371);
			this->LevelDigit->Name = S"LevelDigit";
			this->LevelDigit->Size = System::Drawing::Size(104, 16);
			this->LevelDigit->TabIndex = 3;
			this->LevelDigit->Text = S"__-__";
			this->LevelDigit->TextAlign = System::Drawing::ContentAlignment::MiddleRight;
			// 
			// FreqUp
			// 
			this->FreqUp->Anchor = System::Windows::Forms::AnchorStyles::Bottom;
			this->FreqUp->FlatStyle = System::Windows::Forms::FlatStyle::System;
			this->FreqUp->Location = System::Drawing::Point(448, 339);
			this->FreqUp->Name = S"FreqUp";
			this->FreqUp->Size = System::Drawing::Size(24, 24);
			this->FreqUp->TabIndex = 4;
			this->FreqUp->Text = S"^";
			this->FreqUp->Click += new System::EventHandler(this, &Form1::FreqUp_Click);
			// 
			// FreqDown
			// 
			this->FreqDown->Anchor = System::Windows::Forms::AnchorStyles::Bottom;
			this->FreqDown->FlatStyle = System::Windows::Forms::FlatStyle::System;
			this->FreqDown->Location = System::Drawing::Point(448, 363);
			this->FreqDown->Name = S"FreqDown";
			this->FreqDown->Size = System::Drawing::Size(24, 24);
			this->FreqDown->TabIndex = 5;
			this->FreqDown->Text = S"v";
			this->FreqDown->Click += new System::EventHandler(this, &Form1::FreqDown_Click);
			// 
			// LevelDown
			// 
			this->LevelDown->Anchor = System::Windows::Forms::AnchorStyles::Bottom;
			this->LevelDown->FlatStyle = System::Windows::Forms::FlatStyle::System;
			this->LevelDown->Location = System::Drawing::Point(664, 363);
			this->LevelDown->Name = S"LevelDown";
			this->LevelDown->Size = System::Drawing::Size(24, 24);
			this->LevelDown->TabIndex = 7;
			this->LevelDown->Text = S"v";
			this->LevelDown->Click += new System::EventHandler(this, &Form1::LevelDown_Click);
			// 
			// LevelUp
			// 
			this->LevelUp->Anchor = System::Windows::Forms::AnchorStyles::Bottom;
			this->LevelUp->FlatStyle = System::Windows::Forms::FlatStyle::System;
			this->LevelUp->Location = System::Drawing::Point(664, 339);
			this->LevelUp->Name = S"LevelUp";
			this->LevelUp->Size = System::Drawing::Size(24, 24);
			this->LevelUp->TabIndex = 6;
			this->LevelUp->Text = S"^";
			this->LevelUp->Click += new System::EventHandler(this, &Form1::LevelUp_Click);
			// 
			// FreqIndexLeft
			// 
			this->FreqIndexLeft->Anchor = System::Windows::Forms::AnchorStyles::Bottom;
			this->FreqIndexLeft->FlatStyle = System::Windows::Forms::FlatStyle::System;
			this->FreqIndexLeft->Location = System::Drawing::Point(224, 387);
			this->FreqIndexLeft->Name = S"FreqIndexLeft";
			this->FreqIndexLeft->Size = System::Drawing::Size(64, 16);
			this->FreqIndexLeft->TabIndex = 8;
			this->FreqIndexLeft->Text = S"<";
			this->FreqIndexLeft->Click += new System::EventHandler(this, &Form1::FreqIndexLeft_Click);
			// 
			// FreqIndexRight
			// 
			this->FreqIndexRight->Anchor = System::Windows::Forms::AnchorStyles::Bottom;
			this->FreqIndexRight->FlatStyle = System::Windows::Forms::FlatStyle::System;
			this->FreqIndexRight->Location = System::Drawing::Point(288, 387);
			this->FreqIndexRight->Name = S"FreqIndexRight";
			this->FreqIndexRight->Size = System::Drawing::Size(64, 16);
			this->FreqIndexRight->TabIndex = 9;
			this->FreqIndexRight->Text = S">";
			this->FreqIndexRight->Click += new System::EventHandler(this, &Form1::FreqIndexRight_Click);
			// 
			// LevelIndexRight
			// 
			this->LevelIndexRight->Anchor = System::Windows::Forms::AnchorStyles::Bottom;
			this->LevelIndexRight->FlatStyle = System::Windows::Forms::FlatStyle::System;
			this->LevelIndexRight->Location = System::Drawing::Point(624, 387);
			this->LevelIndexRight->Name = S"LevelIndexRight";
			this->LevelIndexRight->Size = System::Drawing::Size(64, 16);
			this->LevelIndexRight->TabIndex = 11;
			this->LevelIndexRight->Text = S">";
			this->LevelIndexRight->Click += new System::EventHandler(this, &Form1::LevelIndexRight_Click);
			// 
			// LevelIndexLeft
			// 
			this->LevelIndexLeft->Anchor = System::Windows::Forms::AnchorStyles::Bottom;
			this->LevelIndexLeft->FlatStyle = System::Windows::Forms::FlatStyle::System;
			this->LevelIndexLeft->Location = System::Drawing::Point(560, 387);
			this->LevelIndexLeft->Name = S"LevelIndexLeft";
			this->LevelIndexLeft->Size = System::Drawing::Size(64, 16);
			this->LevelIndexLeft->TabIndex = 10;
			this->LevelIndexLeft->Text = S"<";
			this->LevelIndexLeft->Click += new System::EventHandler(this, &Form1::LevelIndexLeft_Click);
			// 
			// label1
			// 
			this->label1->BackColor = System::Drawing::Color::Transparent;
			this->label1->Font = (new System::Drawing::Font(S"Verdana", 18, System::Drawing::FontStyle::Bold, System::Drawing::GraphicsUnit::Point, 
				(System::Byte)0));
			this->label1->Location = System::Drawing::Point(560, 310);
			this->label1->Name = S"label1";
			this->label1->Size = System::Drawing::Size(112, 24);
			this->label1->TabIndex = 12;
			this->label1->Text = S"Level";
			this->label1->TextAlign = System::Drawing::ContentAlignment::MiddleCenter;
			// 
			// label2
			// 
			this->label2->BackColor = System::Drawing::Color::Transparent;
			this->label2->Font = (new System::Drawing::Font(S"Verdana", 18, System::Drawing::FontStyle::Bold, System::Drawing::GraphicsUnit::Point, 
				(System::Byte)0));
			this->label2->Location = System::Drawing::Point(200, 306);
			this->label2->Name = S"label2";
			this->label2->Size = System::Drawing::Size(200, 32);
			this->label2->TabIndex = 13;
			this->label2->Text = S"Frequency";
			this->label2->TextAlign = System::Drawing::ContentAlignment::MiddleCenter;
			// 
			// label3
			// 
			this->label3->BackColor = System::Drawing::Color::Transparent;
			this->label3->Font = (new System::Drawing::Font(S"Verdana", 9.75F, System::Drawing::FontStyle::Bold, System::Drawing::GraphicsUnit::Point, 
				(System::Byte)0));
			this->label3->Location = System::Drawing::Point(344, 280);
			this->label3->Name = S"label3";
			this->label3->Size = System::Drawing::Size(248, 16);
			this->label3->TabIndex = 15;
			this->label3->Text = S"Phase of Recv W.R.T. Source";
			this->label3->TextAlign = System::Drawing::ContentAlignment::MiddleCenter;
			// 
			// label4
			// 
			this->label4->BackColor = System::Drawing::Color::Transparent;
			this->label4->Location = System::Drawing::Point(8, 17);
			this->label4->Name = S"label4";
			this->label4->Size = System::Drawing::Size(232, 56);
			this->label4->TabIndex = 16;
			this->label4->Text = S"Note : amplitude decreases at low frequencies. High-pass pole at 50 KHz. Waveform" 
				S" distortion increases below 25 KHz. Phase meter not usable below 200 KHz.";
			// 
			// analogMeterControl1
			// 
			this->analogMeterControl1->BackColor = System::Drawing::Color::Transparent;
			this->analogMeterControl1->ForeColor = System::Drawing::Color::Transparent;
			this->analogMeterControl1->Location = System::Drawing::Point(344, 48);
			this->analogMeterControl1->Name = S"analogMeterControl1";
			this->analogMeterControl1->Size = System::Drawing::Size(248, 224);
			this->analogMeterControl1->TabIndex = 14;
			// 
			// Form1
			// 
			this->AutoScaleBaseSize = System::Drawing::Size(5, 13);
			this->BackColor = System::Drawing::SystemColors::ControlLight;
			this->ClientSize = System::Drawing::Size(864, 425);
			this->Controls->Add(this->label4);
			this->Controls->Add(this->label3);
			this->Controls->Add(this->analogMeterControl1);
			this->Controls->Add(this->label2);
			this->Controls->Add(this->label1);
			this->Controls->Add(this->LevelIndexRight);
			this->Controls->Add(this->LevelIndexLeft);
			this->Controls->Add(this->FreqIndexRight);
			this->Controls->Add(this->FreqIndexLeft);
			this->Controls->Add(this->LevelDown);
			this->Controls->Add(this->LevelUp);
			this->Controls->Add(this->FreqDown);
			this->Controls->Add(this->FreqUp);
			this->Controls->Add(this->LevelDigit);
			this->Controls->Add(this->FrequencyDigit);
			this->Controls->Add(this->OutputLevel);
			this->Controls->Add(this->frequency);
			this->ForeColor = System::Drawing::SystemColors::ControlText;
			this->Menu = this->mainMenu1;
			this->Name = S"Form1";
			this->Text = S"Note";
			this->FormClosed += new System::Windows::Forms::FormClosedEventHandler(this, &Form1::Form1_Close);
			this->Load += new System::EventHandler(this, &Form1::Form1_Load);
			this->ResumeLayout(false);

		}	
	
private: System::Void SGEN_Initialize(void)
		{
			Fs = new FrequencySelector();
   			StartUpPath = Directory::GetCurrentDirectory();		// Save start-up directory path
			Fs->Load(StartUpPath);								// initialize FrequencySelector
																// from saved Reference Parameters

			// timer to periodically poll VNA for phase reading
			timer1 = new System::Windows::Forms::Timer();
			timer1->Tick += new EventHandler(this, &Form1::TimerEventProcessor);
			timer1->Interval = 1000;		// 1 second intervals

			//rand = new Random();

			TxFrequency = 10000000.0;
			TxLevel = 0.0;

	// Open the target 8051 binary file, and prepare to download it

			FileStream* fs;
			Byte Codebuffer __gc[];
			int Length;

			OpenFileDialog* infile = new OpenFileDialog();

	// Hard code the name of the code to download to the VNA
			infile->FileName = String::Concat(StartUpPath, S"\\SGEN_target.bin");

			try
			{
				fs = new FileStream(infile->FileName,FileMode::Open, FileAccess::Read);
			}
			catch(System::Exception* pe)
			{
				MessageBox::Show(pe->Message,S"Error");
				Environment::Exit(1);
			}
			
			Length = static_cast<int>(fs->Length);

			if (Length < 8196)
				Codebuffer = new Byte __gc[8196];
			else
			{
				MessageBox::Show(S" Download file too long",S"Error");
				Environment::Exit(1);
			}

			fs->Read(Codebuffer, 0, Length);
			fs->Close();

			VNA = new VNADevice;
			if (VNA->State == 1)		// EZUSBDRV found our device handle
			{
				bool result;

				result = VNA->Stop();				// Make sure 8051 is halted
				if (!result)
					MessageBox::Show(S"VNA Device Error: Stop",S"Error");

				result = VNA->Download(Codebuffer, Length, 0);
				if (!result)
					MessageBox::Show(S"VNA Device Error: Download",S"Error");

				result = VNA->Start();				// Start the 8051 processor
				if (!result)
					MessageBox::Show(S"VNA Device Error: Start",S"Error");

				for (int k=0; k<3; k++)
				{
					result = VNA->Init();				// Build VNA descriptor tables, get pipes
					if (!result)
						MessageBox::Show(S"VNA Device Error: Init",S"Error");
				};

			}
			else
			{
				MessageBox::Show(S"Cannot open VNA Device",S"Error");
				Environment::Exit(1);
			}
		
			// Update the VNA PLL multiplier value
			
			SGEN_TXBUFFER* tbuf = new SGEN_TXBUFFER;
			SGEN_RXBUFFER* rbuf = new SGEN_RXBUFFER;
			tbuf->PLLmult = Fs->refMult + 64;			// Makes sure PLL range bit is set high
			tbuf->ReplyType = SGEN_REPLYTYPE_SETPLL;
			VNA->WriteRead(tbuf, rbuf);

			// update the VNA Tx Frequency and Tx Level
			UpdateSGEN();

			timer1->Start();							// start timer for gathering phase samples

		}	
private: System::Void Form1_Load(System::Object *  sender, System::EventArgs *  e)
		 {
			txLevel = 0;				// default to 0 dbm
			plotTitle = S"";			// Start with no title. User can give us one if desired

			// load numeric indicators with text

			frequency->Text = TxFrequency.ToString("N8");
			OutputLevel->Text = TxLevel.ToString("N2");

			// set currently active digit selector positions
			FrequencyIndex = 6;
			LevelIndex = 1;

			// load Digit controllers with text
			FrequencyDigit->Text = FreqDigitStr[FrequencyIndex];
			LevelDigit->Text = S"-___";

			// Add paint event handler
			Paint += new PaintEventHandler(this, &Form1::Form_Paint);
			// Add resize event handler
			Resize += new EventHandler(this, &Form1::Form_Resize);

		 }
private: System::Void Form_Paint(Object* pSender, PaintEventArgs* pe)
	{
		Form_Render(pe->Graphics, ClientSize);
	}

private: System::Void Form_Resize(Object* pSender, EventArgs* pe)
	{
		Graphics* pg = CreateGraphics();
		Form_Render(pg, ClientSize);
		pg->Dispose();
	}
private: System::Void Form_Render(Graphics* gr, System::Drawing::Size size)
{
    // Resize & Paint give us their size.
    // Convert the size to a rectangle with 0,0 as the (Top,Left) origin
    // then call the Render with the resultant rectangle.
    // (PrintPage calls Render directly with a rectangle region).
    rect.set_Size(size);
    rect.set_X(0);
    rect.set_Y(0);
    Form_Render(gr, rect);
}


private: System::Void Form_Render(Graphics* gr, Rectangle rect)		// rect is our clientsize
	{
		#define LEFTMARGIN	150			// margin between form window and scope display
		#define RIGHTMARGIN 42
		#define TOPMARGIN	15
		#define BOTMARGIN	130

		// clear the entire client window
		gr->Clear(Color::FromKnownColor(KnownColor::Control));
	}

private: System::Void SelRefmenuItem_Click(System::Object *  sender, System::EventArgs *  e)
		{
			ReferenceSelection * Rs = new ReferenceSelection(VNA, Fs, StartUpPath);
			if(Rs->ShowDialog() == DialogResult::OK)
			{
				// Update the VNA PLL multiplier value
			
				SGEN_TXBUFFER* tbuf = new SGEN_TXBUFFER;
				SGEN_RXBUFFER* rbuf = new SGEN_RXBUFFER;

				tbuf->PLLmult = Fs->refMult + 64;			// Makes sure PLL range bit is set high
				tbuf->ReplyType = SGEN_REPLYTYPE_SETPLL;

				VNA->WriteRead(tbuf, rbuf);
			}

			// update the VNA Tx frequency and level
			UpdateSGEN();
		}

private: System::Void frequency_DoubleClick(System::Object *  sender, System::EventArgs *  e)
		 {
				// pop up a direct entry window
				NumEntBox* nb = new NumEntBox();
				nb->set_NumericValue(TxFrequency);
				if (nb->ShowDialog() == DialogResult::OK)
				{
					double tempF = nb->get_NumericValue();
					// check for legitimate values for startF
					double Maxf = Fs->fClk/2.0;
					if (tempF >= 2000.0 && tempF <= Maxf)
					{
						TxFrequency = tempF;
						frequency->Text = TxFrequency.ToString("N8");
						UpdateSGEN();
					}
					else
						MessageBox::Show(String::Concat(S"Frequency must be between 2,000 and ",
						Maxf.ToString("N0")), "Out of Range");
				}
		 }

private: System::Void OutputLevel_DoubleClick(System::Object *  sender, System::EventArgs *  e)
		 {
				// pop up a direct entry window
				NumEntBox* nb = new NumEntBox();
				nb->set_NumericValue(TxLevel);
				if (nb->ShowDialog() == DialogResult::OK)
				{
					float tempF = (float)nb->get_NumericValue();
					// check for legitimate values for startF
					if (tempF >= -50.0 && tempF <= 0.0)
					{
						TxLevel = tempF;
						OutputLevel->Text = TxLevel.ToString("N2");
					}
					else
						MessageBox::Show("Output Level must be between 0.0 and -50.0 dB",
							"Out of Range");
				}
		 }

private: System::Void FreqIndexLeft_Click(System::Object *  sender, System::EventArgs *  e)
		 {
			 if(FrequencyIndex < 14)
				 FrequencyIndex++;
			 else
				 FrequencyIndex = 0;

			 FrequencyDigit->Text = FreqDigitStr[FrequencyIndex];
		 }

private: System::Void FreqIndexRight_Click(System::Object *  sender, System::EventArgs *  e)
		 {
			 if (FrequencyIndex > 0)
				 FrequencyIndex--;
			 else
				 FrequencyIndex = 14;

			 FrequencyDigit->Text = FreqDigitStr[FrequencyIndex];
		 }

private: System::Void FreqUp_Click(System::Object *  sender, System::EventArgs *  e)
		 {
			 TxFrequency += Math::Pow(10.0, (FrequencyIndex-6));
			 if (TxFrequency > Fs->fClk/2.0)
				 TxFrequency = Fs->fClk/2.0;
			 TxFrequency = Fs->DDSToFreq(Fs->FreqToDDS(TxFrequency));
			 frequency->Text = TxFrequency.ToString("N8");
			 UpdateSGEN();
		 }

private: System::Void FreqDown_Click(System::Object *  sender, System::EventArgs *  e)
		 {
			 TxFrequency -= Math::Pow(10.0, (FrequencyIndex-6));
			 if (TxFrequency < 2000.0)
				 TxFrequency = 2000.0;
			 TxFrequency = Fs->DDSToFreq(Fs->FreqToDDS(TxFrequency));
			 frequency->Text = TxFrequency.ToString("N8");
			 UpdateSGEN();
		 }

private: System::Void LevelIndexLeft_Click(System::Object *  sender, System::EventArgs *  e)
		 {
			 if (LevelIndex == 2)
			 {
				 LevelIndex = 0;
				 LevelDigit->Text = S"-_";
			 }
			 else if (LevelIndex == 1)
			 {
				 LevelIndex = 2;
				 LevelDigit->Text = S"-____";
			 }
			 else if (LevelIndex == 0)
			 {
				LevelIndex = 1;
				LevelDigit->Text = S"-___";
			 }
		 }

private: System::Void LevelIndexRight_Click(System::Object *  sender, System::EventArgs *  e)
		 {
			 if (LevelIndex == 0)
			 {
				LevelIndex = 2;
				LevelDigit->Text = S"-____";
			 }
			 else if (LevelIndex == 1)
			 {
                 LevelIndex = 0;
				 LevelDigit->Text = S"-_";
			 }
			 else if (LevelIndex == 2)
			 {
				 LevelIndex = 1;
				 LevelDigit->Text = S"-___";
			 }
		 }

private: System::Void LevelUp_Click(System::Object *  sender, System::EventArgs *  e)
		 {
			 TxLevel += (float)Math::Pow(10.0, (LevelIndex-1));
			 if (TxLevel > 0.0)
				 TxLevel = 0.0;
			 TxLevel = dBFromLinear(TxLevLinear(TxLevel));
			 OutputLevel->Text = TxLevel.ToString("N2");
			 UpdateSGEN();
		 }

private: System::Void LevelDown_Click(System::Object *  sender, System::EventArgs *  e)
		 {
			 TxLevel -= (float)Math::Pow(10.0, (LevelIndex-1));
			 if (TxLevel < -50.0)
				 TxLevel = -50.0;
			 TxLevel = dBFromLinear(TxLevLinear(TxLevel));
			 OutputLevel->Text = TxLevel.ToString("N2");
			 UpdateSGEN();
		 }

private: System::Void TimerEventProcessor(Object* sender, EventArgs* e)
{
		// one-second timer has gone off - get a phase reading and update the AnalogMeter

			 SGEN_TXBUFFER* tbuf = new SGEN_TXBUFFER;
			 SGEN_RXBUFFER* rbuf = new SGEN_RXBUFFER;

			 tbuf->ReplyType = SGEN_REPLYTYPE_QUERY;	// Don't change Freq or Ampl of generator

			 VNA->WriteRead(tbuf, rbuf);

			 float phase = IQtoDegrees(rbuf->TranPI, rbuf->TranPQ);

			 phase += 180.0;			// offset phase for transmission measurements

			 if (phase > 180.0)			// fixup any phase wrap
				phase -= 360.0;
			 if (phase <= -180.0)
				phase += 360.0;

			 analogMeterControl1->MeterValue = (int)phase;
			 analogMeterControl1->Invalidate();

}


private: System::Void UpdateSGEN(System::Void)
		 {
			 // Send & Receive a USB frame to the VNA updating the TxFrequency and OutputLevel

			 SGEN_TXBUFFER* tbuf = new SGEN_TXBUFFER;
			 SGEN_RXBUFFER* rbuf = new SGEN_RXBUFFER;

			 tbuf->TxAccum = Fs->FreqToDDSCorr(TxFrequency);
			 tbuf->IDAClevel = TxLevLinear(TxLevel);
			 tbuf->ReplyType = SGEN_REPLYTYPE_SETFREQ;

			 VNA->WriteRead(tbuf, rbuf);


		 }

private: System::Void menuItem2_Click(System::Object*  sender, System::EventArgs*  e)
		 {
  			 Fs->Save(StartUpPath);		// Save the values to disk
			 Environment::Exit(0);
		 }


private: System::Void Form1_Close(System::Object*  sender, System::Windows::Forms::FormClosedEventArgs*  e)
		 {
  			 Fs->Save(StartUpPath);		// Save the values to disk
			 Environment::Exit(0);
		 }
}; // end class FORM1
} // end namespace SGENR1


